@@ -1,6 +1,6 @@
 from typing import List, Dict, Callable, Tuple, Union
 from mistql.runtime_value import RuntimeValue, RuntimeValueType, assert_type
-from mistql.expression import Expression
+from mistql.expression import BaseExpression
 from mistql.stack import Stack
 from mistql.expression import RefExpression
 from mistql.stack import add_runtime_value_to_stack
@@ -8,11 +8,15 @@ import re
 from functools import cmp_to_key
 import statistics
 
-from mistql.exceptions import MistQLRuntimeError, MistQLTypeError
+from mistql.exceptions import (
+    MistQLRuntimeError,
+    MistQLTypeError,
+    OpenAnIssueIfYouGetThisError,
+)
 
 
-Args = List[Expression]
-Exec = Callable[[Expression, Stack], RuntimeValue]
+Args = List[BaseExpression]
+Exec = Callable[[BaseExpression, Stack], RuntimeValue]
 
 FunctionDefinitionType = Callable[
     [Args, Stack, Exec],
@@ -32,7 +36,9 @@ def builtin(name: str, min_args: int, max_args: Union[None, int] = None):
         def wrapped(arguments: Args, stack: Stack, exec: Exec):
             if not min_args < 0 and len(arguments) < min_args:
                 raise MistQLRuntimeError(f"{name} takes at least {min_args} arguments")
-            if not max_args < 0 and len(arguments) > max_args:
+            if (max_args is not None) and (
+                not max_args < 0 and len(arguments) > max_args
+            ):
                 raise MistQLRuntimeError(f"{name} takes at most {max_args} arguments")
             return fn(arguments, stack, exec)
 
@@ -164,8 +170,8 @@ def dot(arguments: Args, stack: Stack, exec: Exec) -> RuntimeValue:
     left = exec(arguments[0], stack)
     right = arguments[1]
     if not isinstance(right, RefExpression):
-        raise MistQLRuntimeError(f"dot: RHS of the dot operator is not a ref")
-    return left.access(right.name)
+        raise MistQLRuntimeError("dot: RHS of the dot operator is not a ref")
+    return _index_single(RuntimeValue.of(right.name), left)
 
 
 @builtin("map", 2)
@@ -278,11 +284,11 @@ def _index_double(
     if index_two.type == RVT.Null:
         index_two = RuntimeValue.of(len(operand.value))
     if index_one.type != RVT.Number or index_two.type != RVT.Number:
-        raise MistQLRuntimeError(f"index: Non-numbers cannot be used on arrays")
+        raise MistQLRuntimeError("index: Non-numbers cannot be used on arrays")
     index_one_num = index_one.value
     index_two_num = index_two.value
     if index_one_num % 1 != 0 or index_two_num % 1 != 0:
-        raise MistQLRuntimeError(f"index: Non-integers cannot be used on arrays")
+        raise MistQLRuntimeError("index: Non-integers cannot be used on arrays")
     if index_one_num < 0:
         index_one_num = len(operand.value) + index_one_num
     if index_two_num < 0:
@@ -295,14 +301,19 @@ def _index_single(index: RuntimeValue, operand: RuntimeValue):
         assert_type(index, RVT.Number)
         index_num = index.value
         if index_num % 1 != 0:
-            raise MistQLRuntimeError(f"index: Non-integers cannot be used on arrays")
+            raise MistQLRuntimeError("index: Non-integers cannot be used on arrays")
         if index_num < 0:
             index_num = len(operand.value) + index_num
         if index_num < 0 or index_num >= len(operand.value):
             return RuntimeValue.of(None)
         return RuntimeValue.of(operand.value[int(index_num)])
+    elif operand.type == RVT.Object:
+        return operand.access(assert_type(index, RVT.String).value)
+    elif operand.type == RVT.Null:
+        assert_type(index, {RVT.Number, RVT.String})
+        return RuntimeValue.of(None)
     else:
-        return operand.access(index.to_string())
+        raise MistQLRuntimeError(f"index: Cannot index {operand.type}")
 
 
 @builtin("index", 2, 3)
@@ -358,12 +369,6 @@ def regex(arguments: Args, stack: Stack, exec: Exec) -> RuntimeValue:
     )
 
 
-@builtin("stringjoin", 1)
-def stringjoin(arguments: Args, stack: Stack, exec: Exec) -> RuntimeValue:
-    arg = assert_type(exec(arguments[0], stack), RVT.Array)
-    return RuntimeValue.of("".join(x.to_string() for x in arg.value))
-
-
 @builtin("sort", 1)
 def sort(arguments: Args, stack: Stack, exec: Exec) -> RuntimeValue:
     arg = assert_type(exec(arguments[0], stack), RVT.Array)
@@ -419,7 +424,7 @@ def values(arguments: Args, stack: Stack, exec: Exec) -> RuntimeValue:
 def groupby(arguments: Args, stack: Stack, exec: Exec) -> RuntimeValue:
     target = assert_type(exec(arguments[1], stack), RVT.Array)
     mut = arguments[0]
-    groups = {}
+    groups: Dict[str, List[RuntimeValue]] = {}
     for item in target.value:
         key = exec(mut, add_runtime_value_to_stack(item, stack)).to_string()
         if key not in groups:
@@ -470,6 +475,9 @@ def match(arguments: Args, stack: Stack, exec: Exec) -> RuntimeValue:
     elif pattern.type == RVT.String:
         compiled = re.compile(pattern.value)
         return RuntimeValue.of(bool(compiled.search(target.value)))
+    raise OpenAnIssueIfYouGetThisError(
+        "Unexpectedly reaching end of function in match call."
+    )
 
 
 @builtin("=~", 2)
@@ -493,6 +501,9 @@ def replace(arguments: Args, stack: Stack, exec: Exec) -> RuntimeValue:
         return RuntimeValue.of(
             target.value.replace(pattern.value, replacement.value, 1)
         )
+    raise OpenAnIssueIfYouGetThisError(
+        "Unexpectedly reaching end of function in match call."
+    )
 
 
 @builtin("split", 2)
@@ -506,6 +517,9 @@ def split(arguments: Args, stack: Stack, exec: Exec) -> RuntimeValue:
         return RuntimeValue.of(target.value.split(separator))
     elif delimiter.type == RVT.Regex:
         return RuntimeValue.of(list(delimiter.value.split(target.value)))
+    raise OpenAnIssueIfYouGetThisError(
+        "Unexpectedly reaching end of function in match call."
+    )
 
 
 @builtin("stringjoin", 2)
@@ -524,6 +538,7 @@ def sum(arguments: Args, stack: Stack, exec: Exec) -> RuntimeValue:
         total += assert_type(entry, RVT.Number).value
     return RuntimeValue.of(total)
 
+
 @builtin("summarize", 1)
 def summarize(arguments: Args, stack: Stack, exec: Exec) -> RuntimeValue:
     target = assert_type(exec(arguments[0], stack), RVT.Array)
@@ -581,5 +596,3 @@ def flatten(arguments: Args, stack: Stack, exec: Exec) -> RuntimeValue:
     for entry in target.value:
         result.extend(assert_type(entry, RVT.Array).value)
     return RuntimeValue.of(result)
-
-
